package com.example.duplicate.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.example.duplicate.controller.param.*;
import com.example.duplicate.controller.vo.ArchiveMasterVo;
import com.example.duplicate.controller.vo.ArchiveOtherExtVo;
import com.example.duplicate.infrastructure.dao.ArchiveDetailMapper;
import com.example.duplicate.infrastructure.dao.ArchiveMasterMapper;
import com.example.duplicate.infrastructure.dao.ArchiveOtherExtMapper;
import com.example.duplicate.service.TaskService;
import com.example.utils.CommonResult;
import com.example.utils.RedisMq;
import com.example.utils.SnowflakeIdWorker;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * @ClassName TaskServiceImpl
 * @Description 任务实现类
 * @Author linjj
 * @Date 2024/8/5 14:44
 * @Version 1.0
 */
@Service
@Slf4j
public class TaskServiceImpl implements TaskService {

    @Autowired(required = false)
    private RedisMq redisMq;

    private String redisKey = "docus:task:topic_collect_%s_queue:%s";
    @Value("${repairCollectIds}")
    private String repairCollectIds;
    @Autowired
    private ArchiveMasterMapper archiveMasterMapper;
    @Autowired
    private ArchiveOtherExtMapper archiveOtherExtMapper;
    @Autowired
    private ArchiveDetailMapper archiveDetailMapper;
    @Autowired
    private TaskService taskService;

    @Value("${savePath}")
    private String savePath;

    @Override
    public CommonResult GetTask(String collectId) {
        String message = "";
        for (int i = 10; i >= 1; i--) {
            String key = String.format(redisKey, collectId, i);
            message = redisMq.pop(key);
            if (!StringUtils.isEmpty(message) && !message.equals("null")) {
                break;
            }
        }
        if (StringUtils.isEmpty(message) || message.equals("null")) {
            return CommonResult.failed("暂无队列数据");
        } else {
            PrintParam data = JSON.parseObject(message, PrintParam.class);
            log.info("采集器id为" + collectId + "的队列消费信息:" + data);
            return CommonResult.success(data);
        }
    }

    @Override
    public CommonResult repairTask(String masterId) {
        PrintParam printParam = new PrintParam();
        //任务表入参
        ArchiveOtherExtDto dto = new ArchiveOtherExtDto();
        //任务生成时间
        Date date = new Date();
        SimpleDateFormat SycTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //补偿所有采集器id集合
        List<String> collectIds = Arrays.asList(repairCollectIds.split(","));
        //获取当前患者基础信息
        List<ArchiveMasterVo> archiveMasters = archiveMasterMapper.getArchiveMasterByMasterId(masterId);
        if (CollectionUtils.isEmpty(archiveMasters)) {
            log.info("补偿所有采集查询不到masterId为:" + masterId + "的患者住院信息补偿失败");
            return CommonResult.failed("补偿所有采集查询不到masterId为:" + masterId + "的患者住院信息补偿失败");
        }
        for (String collectId : collectIds) {
            try {
                //查询当前患者要补偿采集器是否存在任务，不存在时新增任务
                List<ArchiveOtherExtVo> others = archiveOtherExtMapper.getOtherByMidAndFlag(archiveMasters.get(0).getId(), Integer.parseInt(collectId));
                if (CollectionUtils.isEmpty(others)) {
                    //任务表新增
                    dto.setId(SnowflakeIdWorker.idWorker.nextId());
                    dto.setSycTime(SycTime.format(date));
                    dto.setOtherID("0");
                    dto.setSysFlag(Integer.parseInt(collectId));
                    dto.setSysUpdateTime(SycTime.format(date));
                    dto.setJzh(archiveMasters.get(0).getPatientId());
                    dto.setZyh(archiveMasters.get(0).getInpNo());
                    dto.setStatusFlag(0);
                    dto.setMID(archiveMasters.get(0).getId());
                    if (collectId.equals("8")) {
                        dto.setC1("长临医嘱");
                    }
                    if (collectId.equals("1")) {
                        dto.setC1("护理记录");
                    }
                    archiveOtherExtMapper.addOther(dto);
                }
                GenerateQueue(archiveMasters.get(0), printParam, 10, collectId);
            } catch (Exception e) {
                log.error("masterId为:" + masterId + "的患者补偿所有采集器失败", e.getMessage(), e);
            }
        }
        log.info("masterId为:" + masterId + "的患者所有采集器补偿成功");
        return CommonResult.success("masterId为:" + masterId + "的患者所有采集器补偿成功");
    }

    @Override
    public CommonResult repairTaskByCollectId(String masterId, String collectId) {
        PrintParam printParam = new PrintParam();
        //任务表入参
        ArchiveOtherExtDto dto = new ArchiveOtherExtDto();
        //任务生成时间
        Date date = new Date();
        SimpleDateFormat SycTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //获取当前患者基础信息
        List<ArchiveMasterVo> archiveMasters = archiveMasterMapper.getArchiveMasterByMasterId(masterId);
        if (CollectionUtils.isEmpty(archiveMasters)) {
            log.info("补偿单个查询不到masterId为:" + masterId + "的患者住院信息补偿失败");
            return CommonResult.failed("补偿单个查询不到masterId为:" + masterId + "的患者住院信息补偿失败");
        }
        //查询任务表数据，是否存在，不存在时任务表新增，存在时将任务状态改为0
        List<ArchiveOtherExtVo> others = archiveOtherExtMapper.getOtherByMidAndFlag(masterId, Integer.parseInt(collectId));
        if (CollectionUtils.isEmpty(others)) {
            //任务表新增
            dto.setId(SnowflakeIdWorker.idWorker.nextId());
            dto.setSycTime(SycTime.format(date));
            dto.setOtherID("0");
            dto.setSysFlag(Integer.parseInt(collectId));
            dto.setSysUpdateTime(SycTime.format(date));
            dto.setJzh(archiveMasters.get(0).getPatientId());
            dto.setZyh(archiveMasters.get(0).getInpNo());
            dto.setStatusFlag(0);
            dto.setMID(archiveMasters.get(0).getId());
            if (collectId.equals("8")) {
                dto.setC1("长临医嘱");
            }
            if (collectId.equals("1")) {
                dto.setC1("护理记录");
            }
            archiveOtherExtMapper.addOther(dto);
        } else {
            archiveOtherExtMapper.updateStatic(masterId, collectId);
        }
        GenerateQueue(archiveMasters.get(0), printParam, 10, collectId);
        log.info("补偿患者单个采集器成功masterId为:" + masterId + "的患者住院信息补偿失败");
        return CommonResult.success("补偿患者单个采集器成功");
    }

    @Override
    public CommonResult delFileBySource(String masterId, String collectId) {
        try {
            //根据患者masterId，collectId删除
            boolean stactic = archiveDetailMapper.delFileBySource(masterId, collectId);
            log.info("删除接口调用");
            if (stactic) {
                return CommonResult.success("删除成功,masterId:" + masterId+"删除分类："+collectId);
            }
        } catch (Exception e) {
            return CommonResult.failed("删除失败,masterId:" + masterId+"删除分类："+collectId);
        }
        return CommonResult.failed("删除失败,masterId:" + masterId+"删除分类："+collectId);
    }

    @Override
    public CommonResult updateTask(updateTaskDto dto) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String format = dateFormat.format(new Date());
        log.info(format + "本次维护任务表传输参数为:" + JSONObject.toJSONString(dto));
        if (StringUtils.isBlank(dto.getMid())) {
            log.info(format + "Mid不能为空！");
            log.info(dto.getMid());
            return CommonResult.failed("Mid名不能为空");
        }
        if (StringUtils.isBlank(dto.getStartTime())) {
            log.info(format + "开始时间不能为空！");
            return CommonResult.failed("开始时间不能为空");
        }
        if (StringUtils.isBlank(dto.getEndTime())) {
            log.info(format + "结束时间不能为空！");
            return CommonResult.failed("结束时间不能为空");
        }
        if (StringUtils.isBlank(String.valueOf(dto.getStatusFlag()))) {
            log.info(format + "任务状态不能为空！");
            return CommonResult.failed("任务状态不能为空");
        }
        if (StringUtils.isBlank(dto.getSycObj())) {
            log.info(format + "SycObj患者信息不能为空！");
            return CommonResult.failed("SycObj患者信息不能为空");
        }
        if (StringUtils.isBlank(dto.getPinResult())) {
            log.info(format + "备注信息不能为空！");
            return CommonResult.failed("备注信息不能为空");
        }
        if (StringUtils.isBlank(String.valueOf(dto.getSysFlag()))) {
            log.info(format + "任务分类不能为空！");
            return CommonResult.failed("任务分类不能为空");
        }
        if (archiveOtherExtMapper.updateByMid(dto)) {
            return CommonResult.success("id为:" + dto.getMid() + "任务表维护成功");
        }
        return CommonResult.success("id为:" + dto.getMid() + "任务表维护失败");
    }

    @Override
    public Boolean fileUpload(MultipartFile file, FileUpload dto) {
        try {
            //使用yyyyMMddHHmmssSSS格式作为文件名
            Date date = new Date();
            SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmssSSS");
            String newDate = format.format(date);
            //文件更新时间
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            //文件保存目录
            String filePathdir = savePath + File.separatorChar + dto.getMasterid();
            File filePath = new File(filePathdir);
            //判断文件夹是否存在不存在创建文件夹
            if (!filePath.exists()) {
                filePath.mkdirs();
            }
            //文件地址
            String saveFilePath = filePathdir + File.separatorChar + newDate + ".pdf";
            try {
                file.transferTo(new File(saveFilePath));
            } catch (IOException e) {
                log.info("保存文件失败流水号：" + dto.getSubassort());
                return false;
            }
            try {
                //判断文件表是否存在流水号存则更新
                List<ArchiveDetailDto> archiveDetailList = archiveDetailMapper.getDetailBySerialNumber(dto.getSoleKey());
                if (!CollectionUtils.isEmpty(archiveDetailList)) {
                    log.info("保存文件接口存在删除记录：" + dto.getSubassort());
                    for (ArchiveDetailDto details : archiveDetailList) {
                        //存在删除对应记录
                        if (archiveDetailMapper.deleteByPrimaryKey(details.getId())) {
                            log.info("保存文件接口存在删除记录：" + dto.getSubassort());
                            File file1 = new File(details.getPdfPath());
                            try {
                                file1.delete(); // 删除照片
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }
                //保存文件表
                ArchiveDetailDto archiveDetail = new ArchiveDetailDto();
                archiveDetail.setId(newDate);
                archiveDetail.setPdfPath(saveFilePath);
                archiveDetail.setMasterid(dto.getMasterid());
                archiveDetail.setUploaddatetime(dateFormat.format(new Date()));
                archiveDetail.setAssortid(dto.getAssortid());
                archiveDetail.setSource(dto.getSource());
                archiveDetail.setSubassort(dto.getSubassort());
                archiveDetail.setTitle(dto.getTitle());
                archiveDetail.setFlag("0");
                archiveDetail.setC1(dto.getC1());
                archiveDetail.setSoleKey(dto.getSoleKey());
                if (archiveDetailMapper.insertSel(archiveDetail)) {
                    return true;
                }
            } catch (Exception e) {
                log.error("异常处理：" + e.getMessage(),e);
                return false;
            }
        } catch (Exception e) {
            log.error("异常处理：" + e.getMessage(),e);
        }
        return false;
    }

    @Override
    public CommonResult compensate() {
        //查询所有需要补偿患者id
        List<String> ids = archiveMasterMapper.getMasterid();
        for (String id : ids) {
            taskService.repairTask(id);
        }
        return CommonResult.success("补偿7月份患者医嘱成功");
    }


    //生成队列消息
    private void GenerateQueue(ArchiveMasterVo task, PrintParam data, Integer ispriority, String collectid) {
        data.setId(task.getId());
        data.setInpNo(task.getInpNo());
        data.setVisitId(task.getVisitId());
        data.setDeptName(task.getDeptName());
        data.setDischargeDateTime(task.getDischargeDateTime());
        data.setName(task.getName());
        data.setAdmissionDateTime(task.getAdmissionDateTime());
        String key = String.format(redisKey, collectid, ispriority);
        redisMq.push(key, JSON.toJSONString(data, SerializerFeature.WriteDateUseDateFormat, SerializerFeature.WriteMapNullValue));
    }
}


